##############
# FIRST PART #
##############

import numpy as np
import matplotlib
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D

from fft_scripts import *

# mesh size
N = 512

# produces the field and writes it on a file
(X,Y,field)=produce_field(N)
field.tofile('field.dat')

# plots the field
figure = plt.figure()
myplot = figure.add_subplot(projection='3d')
surf = myplot.plot_surface(X, Y, field)
myplot.set_zlim(0, 1.5)
plt.show()
plt.savefig('density.png')


#####################
# END OF FIRST PART #
#####################

# HERE YOU SHOUD RUN:
# > ./fft.x 512 0

###############
# SECOND PART #
###############

# loads the transform
transform=np.fromfile('transform.dat').reshape(N,N,2)

# Symmetry: values at the "corners" are real numbers
filter=(transform[:,:,1]==0)
print("At these positions the field is real:")
print(X[filter])
print(Y[filter])

# Symmetry: F(-k) = *F(k)
ii=np.random.randint(N/2)
jj=np.random.randint(N/2)
print("Check of symmetry of transformed field")
print("Choosen point: ",ii,jj)
print("F(k):  ",transform[ii,jj])
print("F(-k): ",transform[-ii,-jj])

# checks that the back transform is correct
result=np.fromfile('back.dat').reshape(N,N)
residual=result-field
print("Largest residual wrt fft: ",np.amax(np.abs(residual)))
print("Average residual wrt fft: ",np.average(np.abs(residual)))

######################
# END OF SECOND PART #
######################

# HERE YOU SHOULD RUN:
# > ./fft.x 512 1
# > ./fft.x 512 2

##############
# THRID PART #
##############

# derivatives

# from numpy gradient function
(xg,yg)=np.gradient(field)
xg *= 512.0
yg *= 512.0

xder=np.fromfile('x_derivative.dat').reshape(N,N)
xanalytic=xderivative(N)

# plots the x-derivative of the field
figure = plt.figure()
myplot = figure.add_subplot(projection='3d')
myplot.set_title('x analytic')
surf = myplot.plot_surface(X, Y, xanalytic)
myplot.set_zlim(-10, 10)
plt.show()
plt.savefig('Xder_analytic.png')

figure = plt.figure()
myplot = figure.add_subplot(projection='3d')
myplot.set_title('x fourier')
surf = myplot.plot_surface(X, Y, xder)
myplot.set_zlim(-10, 10)
plt.show()
plt.savefig('Xder_fft.png')

figure = plt.figure()
myplot = figure.add_subplot(projection='3d')
myplot.set_title('x numpy gradient')
surf = myplot.plot_surface(X, Y, xg)
myplot.set_zlim(-10, 10)
plt.show()
plt.savefig('Xder_gradient.png')

residual=(xder-xanalytic)
print("Largest residual wrt fft: ",np.amax(np.abs(residual)))
print("Average residual wrt fft: ",np.average(np.abs(residual)))

figure = plt.figure()
myplot = figure.add_subplot(projection='3d')
myplot.set_title('x residual wrt fft')
surf = myplot.plot_surface(X, Y, residual)
myplot.set_zlim(-0.01, 0.01)
plt.show()
plt.savefig('Xder_residuals_fft.png')

residual=(xg-xanalytic)
print("Largest residual wrt gradient: ",np.amax(np.abs(residual)))
print("Average residual wrt gradient: ",np.average(np.abs(residual)))

figure = plt.figure()
myplot = figure.add_subplot(projection='3d')
myplot.set_title('x residual wrt gradient')
surf = myplot.plot_surface(X, Y, residual)
myplot.set_zlim(-0.01, 0.01)
plt.show()
plt.savefig('Xder_residuals_gradient.png')



yder=np.fromfile('y_derivative.dat').reshape(N,N)
yanalytic=yderivative(N)

# plots the y-derivative of the field
figure = plt.figure()
myplot = figure.add_subplot(projection='3d')
myplot.set_title('y analytic')
surf = myplot.plot_surface(X, Y, yanalytic)
myplot.set_zlim(-10, 10)
plt.show()
plt.savefig('Yder_analytic.png')

figure = plt.figure()
myplot = figure.add_subplot(projection='3d')
myplot.set_title('y fourier')
surf = myplot.plot_surface(X, Y, yder)
myplot.set_zlim(-10, 10)
plt.show()
plt.savefig('Yder_fft.png')

figure = plt.figure()
myplot = figure.add_subplot(projection='3d')
myplot.set_title('y numpy gradient')
surf = myplot.plot_surface(X, Y, yg)
myplot.set_zlim(-10, 10)
plt.show()
plt.savefig('Yder_gradient.png')

residual=(yder-yanalytic)
print("Largest residual wrt fft: ",np.amax(np.abs(residual)))
print("Average residual wrt fft: ",np.average(np.abs(residual)))

figure = plt.figure()
myplot = figure.add_subplot(projection='3d')
myplot.set_title('y residual wrt fft')
surf = myplot.plot_surface(X, Y, residual)
myplot.set_zlim(-0.01, 0.01)
plt.show()
plt.savefig('Yder_residuals_fft.png')

residual=(yg-yanalytic)
print("Largest residual wrt gradient: ",np.amax(np.abs(residual)))
print("Average residual wrt gradient: ",np.average(np.abs(residual)))

figure = plt.figure()
myplot = figure.add_subplot(projection='3d')
myplot.set_title('y residual wrt gradient')
surf = myplot.plot_surface(X, Y, residual)
myplot.set_zlim(-0.01, 0.01)
plt.show()
plt.savefig('Yder_residuals_gradient.png')
